#!/bin/bash
#
# vdr-requant.sh - v0.0.42a
#
#
# User settings
# =============
#
# This value should point to your VDR-recording-directory 
# (e.g. '/video0' or '/video')
#
VIDEODIR=/video

# This value (in kB) is used to determine if splitting is
# needed. As of vdr-1.2.6 the maximum value VDR can use is
# 2048 MB (actually VDR itself only uses 2000 MB).
#
SIZELIMIT=2000

# This value moves the original dir to .del to let VDR delete it later
# and moves the result to a new directory named the same as originaldir
#   0 means result will be of year 2014, original will be kept
#   1 means replace orig.with result (mark orig.as deleted but keep on disc)
#   2 means like 1, but mark with ^% instead of %
#   3 means like 2 but keeps original recording
#
REMOVE=0

# Beeping can have the following values:
#   1 means vdr-requant.sh will beep when finnished
#   0 means vdr-requant.sh won't beep when finnished
#
BEEP=1

# This is used as the default factor, if used with vdr -r (or option after)
#
DEFAULT_FACTOR=1.3

##############################################################################
# The values below are only EXPERIMENTAL - you normally don't need to change
# them...

# Nicelevel the started programs will be executed with
#
NICEVALUE=15

# requant-program
#   0 means requant from M2VRequantizer
#   1 means tcrequant from transcode
#
REQUANT=1

# use vdrcutter2
#   0 means do not cut the original, directly requant it
#   1 means use it to cut the original before requanting 
#
CUTTER=0

# EXPERIMENTAL
# try to create transport-stream instead of dvd-compliant output
#   0 means try to create dvd compatible mpeg file
#   1 means try to create a normal VDR recording to be reused by vdrconvert etc.
#
OUTPUTFORMAT=1   # 1 requires udrec_suite installed

# Use bitrate calculation code by mhunstig (which requires bc)
#   0 means don't use it
#   1 means use it
USE_BRCALC=1

# When using target bitrate, if the calculated factor ist smaller than this,
# requant will leave the recording untouched.
#
MIN_FACTOR=1.05

# This defines the maximum factor that will be applied when using
# target bitrate as a parameter.
#
MAX_FACTOR=2

# Choose version of vdrsync
#
VDRSYNC=vdrsync-0.1.2.2-dev2.pl

######################################################################
#
# YOU (HOPEFULLY) NEVER NEED TO CHANGE ANYTHING BELOW :)
#

# Display syntax if not called correctly
#
if [ $# -ne 2 ]
	then
	echo "Usage   : $0 <Requant-Factor(e.g.:1.3)>		<Recording-Dir>"
	if [ $USE_BRCALC -eq 1 ] 
	then
		echo "or      : $0 <video bitrate in kbps(e.g.:10000)>	<Recording-Dir>"
	fi
	echo "or      : $0 after					<Recording-Dir>"
	echo
	echo "Example : $0 1.3 /video0/Mymovie/2003-10-08.00\:15.55.99.rec/"
	echo
	exit 128
else
	if [ "$1" = "after" ]
	then
		FACTOR=$DEFAULT_FACTOR
	else
		if [ $USE_BRCALC -eq 1 ]
		then
			#mhunstig code below
#			if [ $( echo "$1*100000" | bc | awk -F. '{print $1 }') -lt 5000000 ]
			if [ $( echo "$1" | awk -F. '{print $1 }') -lt 50 ]
			then
				FACTOR=$1
			else
#				BRINPUT=$( vdrsync.pl -script-output -i $2/001.vdr | grep Bitrate | head -n 1 | awk -F: '{print $2 }' | sed -n '$ p' )
				BRINPUT=$[ $( tcprobe -i $2/001.vdr 2>/dev/null \
				         | grep kbps | head -n 1 | cut -d = -f4- \
				         | cut -d " " -f1 ) * 1000 ]
				echo input bitrate: $(($BRINPUT/1000)) kbps  ===\>  output bitrate: $1 kbps
				FACTOR=$(echo "scale=5; $BRINPUT/$1/1000" | bc )
				echo calculated factor: $FACTOR
				if [ $( echo "scale=5; 100000*$FACTOR/$MIN_FACTOR" | bc | awk -F. '{print $1 }') -lt 100001 ]
				then
					echo "Calculated factor ist below $MIN_FACTOR, requant will not change anything."
					exit 128
				elif [ $( echo "scale=0; $MAX_FACTOR/$FACTOR" | bc ) -lt 1 ]
				then
					FACTOR=$MAX_FACTOR
					echo "Calculated factor larger than defined maximum factor. Using factor $FACTOR."	
				fi
			fi
		else
			FACTOR=$1
		fi
	fi

	MOVIEDIR=$2
fi

# Check if needed supplementals are there
#
needed_progs="vdrsync.pl tcmplex genindex svdrpsend.pl split at awk sed"
if [ $REQUANT -eq 0 ]
then	needed_progs="requant    $needed_progs"
else	needed_progs="tcrequant  $needed_progs"
fi
if [ $CUTTER -eq 1 ]
then	needed_progs="vdrcutter2 $needed_progs"
fi
if [ $OUTPUTFORMAT -eq 1 ]
then	needed_progs="pes2ts     $needed_progs"
fi
if [ $USE_BRCALC -eq  1 ]
then
	needed_progs="bc	$needed_progs"
	echo "INFO: adding bc to needed programs"
fi
echo -n "DEBUG: scanning progs:"
for prog in $needed_progs
do
	needed_prog=$( which $prog )
	if [ "$needed_prog" = "" ]
	then
		echo ERROR: you need $prog in your path.
		echo which $prog:
		echo $needed_prog
		exit 128
	fi
	if [ -x $needed_prog ] 
	then	echo -n " "$prog
	else	echo ERROR: $prog in your path has to be executable
		echo which $prog:
		echo $needed_prog
		exit 128
	fi
done
echo .

# check if size fits...
#
SPLIT=0		# this will be changed automatically if size > $SIZELIMIT
SIZE=$( du -smL $MOVIEDIR | cut -f1 )
EXPECTEDSIZE=$[ $[ $SIZE / $( echo $FACTOR | sed "s/\.//" ) * 10 ] + 10 ]
FREEONDISK=$( df -m $MOVIEDIR | tail -n1 | awk '{ print $4 }' )
if [ $FREEONDISK -lt $[ $SIZE * 3 ] ]
then
	echo "ERROR: not enough space on disk ($FREEONDISK < $[ $SIZE * 3 ])"
	exit 128
else
	echo "DEBUG: enough space on disk - fine. ($FREEONDISK > $[ $SIZE * 3 ])"
fi
if [ $EXPECTEDSIZE -lt $SIZELIMIT ]
then	echo "DEBUG: $SIZE less than max.($SIZELIMIT) - fine."
else	echo "DEBUG: greater than $SIZELIMIT-needs splitting."
	SPLIT=1
fi

# find out destination dir
#
if echo $MOVIEDIR | egrep "^$VIDEODIR" >/dev/null
then	BASENAME=$( basename $MOVIEDIR )
	BASENAME=$( echo $BASENAME | sed "s/200/201/" )
	DIRNAME=$(  dirname $MOVIEDIR )
	if grep -l 'reduced by' summary.vdr &>/dev/null
	then echo ERROR: already reduced.
	     exit 128
	else echo DEBUG: not yet reduced - fine.
	fi

	RESULTDIR=$DIRNAME/$BASENAME
	echo DEBUG: videodir ok - temporary target is $RESULTDIR.
	if [ $REMOVE -eq 2 -o $REMOVE -eq 3 ]
	then 	echo DEBUG: videodir will be marked with '^'  later.
	fi
else	echo ERROR: recording not in $VIDEODIR
	exit 128
fi
mkdir $RESULTDIR
cd $RESULTDIR

######################################################################
# if vdrcutter is used, do some cutting
#
if [ $CUTTER -eq 1 ]
then	echo
	echo part 0/4 - cutting
	echo
	echo DEBUG: vdrcutter2 $MOVIEDIR/ \> 001.vdr
	vdrcutter2 $MOVIEDIR/ > 001.vdr 2> /dev/null
	if [ $? -eq 0 ] 
	then	echo DEBUG: vdrcutter2 ended successfully.
	else	echo ERROR: vdrcutter2 exited with error.
		exit 128
	fi
fi

######################################################################
# start vdrsync.pl and collect results
#
echo
echo part 1/4 - vdrsync.pl
echo
echo DEBUG: starting: vdrsync.pl
if [ $CUTTER -eq 0 ] 
then	nice -n $NICEVALUE $VDRSYNC $MOVIEDIR \
		&> /tmp/vdr-req-1-vdrsync.log
else	nice -n $NICEVALUE $VDRSYNC $RESULTDIR \
		&> /tmp/vdr-req-1-vdrsync.log
fi
if [ $? -eq 0 ] 
then	echo DEBUG: vdrsync.pl ended successfully.
else	echo ERROR: vdrsync.pl exited with error.
	exit 128
fi

VIDEO=$( ls *mpv )
AUDIO1=$( ls *mpa | head -n1 )
AUDIO2=$( ls *.ac3 )

if [ "$AUDIO1" = "" ] && [ "$AUDIO2" = "" ]
then 	echo ERROR: no audio exists
	exit 128
fi
if [ "$VIDEO" = "" ]
then	echo ERROR: no video exists
	exit 128
fi



######################################################################
echo
echo part 2/4 - requant
echo
if [ $REQUANT -eq 0 ]
then
	echo "DEBUG: starting: requant $FACTOR < $VIDEO > 001.mpv"
	nice -n $NICEVALUE requant $FACTOR < $VIDEO > 001.mpv \
		2>/tmp/vdr-req-2-requant.log
	if [ $? -eq 0 ] 
	then	echo DEBUG: requant ended successfully.
		rm $VIDEO
	else	echo ERROR: requant exited with error.
		exit 128
	fi
else
	echo "DEBUG: starting: tcrequant -f $FACTOR < $VIDEO > 001.mpv"
	nice -n $NICEVALUE tcrequant -f $FACTOR \
		< $VIDEO > 001.mpv 2>/tmp/vdr-req-2-requant.log
	if [ $? -eq 0 ] 
	then	echo DEBUG: tcrequant ended successfully.
		rm $VIDEO
	else	echo ERROR: tcrequant exited with error.
		exit 128
	fi
fi
######################################################################
echo
echo part 3/4 - mplex and split
echo

echo DEBUG: starting: mplex -f 9 -o tmp.vdr 001.mpv $AUDIO1 $AUDIO2
nice -n $NICEVALUE mplex -f 9 -o tmp.vdr 001.mpv $AUDIO1 $AUDIO2 \
	&> /tmp/vdr-req-3-tcmplex.log
if [ $? -eq 0 ] 
then	echo DEBUG: mplex ended successfully.
else	echo ERROR: mplex exited with error.
	exit 128
fi

if [ $OUTPUTFORMAT -eq 1 ]
then	echo DEBUG: pes2ts
nice -n $NICEVALUE java -jar /usr/local/lib/jar/pX.jar -c /usr/local/lib/jar/pX_ts.ini -o . tmp.vdr \
	&> /tmp/vdr-req-3-ts.log
	mv tmp.ts 001.vdr
	rm tmp.vdr
fi


if [ $SPLIT -eq 1 ]
then	CURRSIZE=$( du -sm 001.vdr | cut -f 1 )
	SPLITSIZE=$[ $CURRSIZE / $[ $CURRSIZE / $SIZELIMIT + 1 ] * 1048576 ]
	echo -n "WARNING: splitting also means we are loosing "
	echo "$[ $[ $CURRSIZE / $SIZELIMIT ] + 1 ] frames (if not even GOPs)"
	echo DEBUG: starting: split -b $[ $SPLITSIZE ] 001.vdr
	nice -n $NICEVALUE split -b $[ $SPLITSIZE ] 001.vdr \
		&> /tmp/vdr-req-3-split.log
	if [ $? -eq 0 ] 
	then	echo DEBUG: split ended successfully.
	else	echo ERROR: split exited with error.
		exit 128
	fi
	
	count=0
	for a in x*
	do	count=$[ $count + 1]
		srcfile=$a
		targetfile=$count
		[ -f $srcfile ] && mv $srcfile $targetfile
	done

	(	for a in ?   ; do mv $a 00$a.vdr ; done
		for a in ??  ; do mv $a  0$a.vdr ; done
		for a in ??? ; do mv $a   $a.vdr ; done ) &>/dev/null
else
	echo DEBUG: no splitting needed...
fi

echo 
echo part 4/4 - genindex and cleanup
echo
echo DEBUG: starting: genindex
nice -n $NICEVALUE genindex >/tmp/vdr-req-4-genindex.log
if [ $? -eq 0 ] 
then	echo DEBUG: genindex ended successfully.
else	echo ERROR: genindex exited with error.
	exit 128
fi

# clean up
#
rm *mpv *mpa
[ -f *ac3 ] && rm *ac3
if [ -f $MOVIEDIR/summary.vdr ] 
then   ( echo -n "[reduced by $FACTOR] " ; cat $MOVIEDIR/summary.vdr ) \
	>summary.vdr
else   echo -n "[reduced by $FACTOR] (no summary existed before) " \
	>summary.vdr
fi
touch marks.vdr

if [ $CUTTER -eq 1 ]
then
	base=$( basename $( dirname $MOVIEDIR ) )
	mvdir=$( echo $MOVIEDIR | sed "s/$base/%$base/" )

	mkdir -p $mvdir
	mv $MOVIEDIR/* $mvdir
	rmdir $MOVIEDIR
	MOVIEDIR=$mvdir
	unset mvdir
fi


if [ $REMOVE -eq 1 ]
then
	mv $MOVIEDIR $( echo $MOVIEDIR | sed "s/\.rec/\.del/" )
	mkdir $MOVIEDIR
	mv * $MOVIEDIR
	cd
	rmdir $RESULTDIR
fi

if [ $REMOVE -eq 2 ]
then
	mv $MOVIEDIR $( echo $MOVIEDIR | sed "s/\.rec/\.del/" )
	NEWTARGET=$( echo $MOVIEDIR    | sed "s/%/^%/" )
	mkdir -p $NEWTARGET
	mv *  $NEWTARGET
	cd
	rmdir $RESULTDIR
fi

if [ $REMOVE -eq 3 ]
then
#	mv $MOVIEDIR $( echo $MOVIEDIR | sed "s/\.rec/\.del/" )
	NEWTARGET=$( echo $MOVIEDIR    | sed "s/%/^%/" )

	mkdir -p $NEWTARGET
	mv *  $NEWTARGET
	cd
	rmdir $RESULTDIR
fi

[ -d $VIDEODIR ] && touch $VIDEODIR/.update

echo done.
echo

# beep when finished
#
if [ $BEEP -eq 1 ]
then	echo -e \\\a >/dev/tty0
fi

